package answer

import (
	"context"
	"fmt"
	"github.com/minio/minio-go/v7"
	"net/url"
	"time"
)

const (
	bucketAnswers = "squizanswer"
)

type StorerAnswer struct {
	client *minio.Client
}

func NewAnswerMinio(ctx context.Context, minioClient *minio.Client) (*StorerAnswer, error) {
	if ok, err := minioClient.BucketExists(ctx, bucketAnswers); !ok {
		if err := minioClient.MakeBucket(ctx, bucketAnswers, minio.MakeBucketOptions{}); err != nil {
			return nil, err
		}
	} else if err != nil {
		return nil, err
	}

	return &StorerAnswer{
		client: minioClient,
	}, nil
}

func (s *StorerAnswer) GetAnswerURL(ctx context.Context, quizID string, questionID int64, filename string) (string, error) {
	objectName := fmt.Sprintf("%s/%d/%s", quizID, questionID, filename)

	reqParams := make(url.Values)
	reqParams.Set("response-content-disposition", "attachment")
	url, err := s.client.PresignedGetObject(ctx, bucketAnswers, objectName, time.Hour*1, reqParams)
	if err != nil {
		return "", err
	}

	return url.String(), nil
}
